package be.iminds.ilabt.jfed.restapi.json;

import be.iminds.ilabt.jfed.restapi.json.util.JsonGeniUrnDeserializer;
import be.iminds.ilabt.jfed.restapi.json.util.JsonGeniUrnSerializer;
import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.util.jsonld.JsonLdObjectsMetaData;
import be.iminds.ilabt.util.jsonld.impl.ForeignIdObject;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import java.net.URI;
import java.sql.Timestamp;

/**
 * SpeaksForCredential
 */
@JsonLdObjectsMetaData.JsonLdObjectInfo(
        pathName = "speaksForCredential", builderClass = SpeaksForCredentialBuilder.class,
        uriType = JsonLdObjectsMetaData.UriType.ID_BASED, idType = JsonLdObjectsMetaData.IdType.FOREIGN)
public class SpeaksForCredential extends ForeignIdObject {
    private final String credentialXml;
    private final GeniUrn speaker;
    private final GeniUrn speakingFor;
    private final Timestamp expire;

    @JsonCreator
    public SpeaksForCredential(@JsonProperty("credentialXml") String credentialXml,
                               @JsonProperty("speaker") GeniUrn speaker,
                               @JsonProperty("speakingFor") GeniUrn speakingFor,
                               @JsonProperty("expire") Timestamp expire,
                               @JsonProperty("uri") URI uri) {
        this(credentialXml, speaker, speakingFor, expire, uri,
                JsonLdObjectsMetaData.mustSerializeAsEmbeddedObjectForJacksonConstructor());
    }

    public SpeaksForCredential(String credentialXml,
                               GeniUrn speaker,
                               GeniUrn speakingFor,
                               Timestamp expire,
                               URI uri,
                               boolean serializeInfo_SerializeAsEmbeddedObject) {
        super(uri, serializeInfo_SerializeAsEmbeddedObject);
        this.credentialXml = credentialXml;
        this.speaker = speaker;
        this.speakingFor = speakingFor;
        this.expire = expire;
    }

    @JsonProperty
    public String getCredentialXml() {
        return credentialXml;
    }

    @JsonProperty
    @JsonSerialize(using = JsonGeniUrnSerializer.class)
    @JsonDeserialize(using = JsonGeniUrnDeserializer.class)
    public GeniUrn getSpeaker() {
        return speaker;
    }

    @JsonProperty
    @JsonSerialize(using = JsonGeniUrnSerializer.class)
    @JsonDeserialize(using = JsonGeniUrnDeserializer.class)
    public GeniUrn getSpeakingFor() {
        return speakingFor;
    }

    @JsonProperty
    public Timestamp getExpire() {
        return expire;
    }
}
